<?php

namespace DigitalPoint\Cloudflare\Listener;

class AppPubStartEnd
{
	public static function run(\XF\App $app)
	{
		if ($app->request()->getRoutePath() != 'logout/')
		{
			// Faking a valid CSRF token if the user has valid Sec-Fetch-Site header
			$secFetchSite = $app->request()->getServer('HTTP_SEC_FETCH_SITE', '');
			if ($secFetchSite)
			{
				if ($secFetchSite === 'same-origin' || $secFetchSite === 'same-site')
				{
					$app->request()->set('_xfToken', $app['csrf.token']);
				}
			}
			/*
			 * Faking a valid CSRF token if the user has no Sec-Fetch-Site header, but they have a referrer (Safari was late to the Sec-Fetch-Site game, so this is mostly for Safari < 16.4)
			 *
			 * This is not perfect because XenForo is mutating state for certain GET requests (bad idea).
			 * Logout, cookie consent, style selection, language selection, profile post moderation are all GET requests when they really should be POST requests, but it is what it is...
			 */
			elseif ($app->request()->getReferrer() && strtolower(parse_url($app->request()->getReferrer(), PHP_URL_HOST)) === strtolower(parse_url($app->options()->boardUrl, PHP_URL_HOST)))
			{
				$app->request()->set('_xfToken', $app['csrf.token']);
			}
		}
	}
}


/*
 * What I really want to do is extend XF\Mvc\Controller and use Sec-Fetch-Site header directly...
 * However we can't extend abstract classes, so we are doing the slightly hacky thing in this listener class.
 *
 * This would be ideal (at this point it should probably be in XF core and just use cookie-based CSRF token as a fallback):

class Controller extends XFCP_Controller
{
	public function validateCsrfToken($token = null, &$error = null, $validityPeriod = null)
	{
		$secFetchSite = $this->request->getServer('HTTP_SEC_FETCH_SITE', '');
		if ($secFetchSite)
		{
			if ($secFetchSite === 'same-origin' || $secFetchSite === 'same-site')
			{
				return true;
			}
		}
		elseif ($this->request->getReferrer() && strtolower(parse_url($this->request->getReferrer(), PHP_URL_HOST)) === strtolower(parse_url($this->options()->boardUrl, PHP_URL_HOST)))
		{
			return true;
		}
		return parent::validateCsrfToken($token, $error, $validityPeriod);
	}
}

 */