<?php

namespace DigitalPoint\Cloudflare\Stats;

use XF\Stats\AbstractHandler;

class Cloudflare extends AbstractHandler
{
	public function getStatsTypes()
	{
		return [
			'cloudflare_unique_visitor' => \XF::phrase('unique_visitors'),
			'cloudflare_request' => \XF::phrase('requests'),
			'cloudflare_data_served' => \XF::phrase('data_served_usage'),
			'cloudflare_data_cached' => \XF::phrase('data_cached_usage'),
			'cloudflare_threats' => \XF::phrase('threats'),
			'turnstile_challenge' => \XF::phrase('turnstile_challenges'),
			'turnstile_solve_interactive' => \XF::phrase('turnstile_solves_interactive'),
			'turnstile_solve_noninteractive' => \XF::phrase('turnstile_solves_noninteractive'),
			'turnstile_unsolved' => \XF::phrase('turnstile_unsolved'),
			'r2_operation_class_a' => \XF::phrase('r2_class_a_operations'),
			'r2_operation_class_b' => \XF::phrase('r2_class_b_operations'),
		];
	}

	public function getData($start, $end)
	{
		$return = [];

		// Ignore requests for full rebuild (takes too long and there's also API rate limits in play that would make it fail anyway).
		if ($end - $start > 86400)
		{
			return $return;
		}

		try {
			for ($timestamp = $start; $timestamp <= $end; $timestamp = $timestamp + 86400)
			{
				$stats = $this->app->repository('DigitalPoint\Cloudflare:Cloudflare')->getStats($timestamp);

				foreach ($stats as $name => $counter)
				{
					$return[$name][$timestamp - $timestamp % 86400] = $counter;
				}
			}
		}
		catch(\Exception $e) {}

		return $return;
	}

	public function adjustStatValue($statsType, $counter)
	{
		if ($statsType == 'cloudflare_data_served' || $statsType == 'cloudflare_data_cached')
		{
			return round($counter / 1000000, 2);
		}
		return parent::adjustStatValue($statsType, $counter);
	}
}