<?php

namespace DigitalPoint\Cloudflare\Traits;

trait XF
{
	protected function getClassName($className)
	{
		return \XF::extendClass($className);
	}

	public function option($optionKey)
	{
		$split = explode('.', $optionKey, 3);
		if (count($split) === 1)
		{
			return \XF::options()->{$split[0]};
		}
		elseif(count($split) === 2)
		{
			return \XF::options()->{$split[0]}[$split[1]];
		}
		return \XF::options()->{$split[0]}[$split[1]][$split[2]];
	}

	protected function updateOption($name, $value = '')
	{
		if (is_array($name))
		{
			\XF::repository('XF:Option')->updateOptions($name);
		}
		else
		{
			\XF::repository('XF:Option')->updateOption($name, $value);
		}
	}

	protected function getSiteUrl()
	{
		return \XF::options()->boardUrl;
	}

	protected function isAdmin()
	{
		return \XF::visitor()->is_admin;
	}

	protected function isDebug()
	{
		return \XF::$debugMode;
	}

	protected function time()
	{
		return \XF::$time;
	}

	protected function request($method, $url, $options)
	{
		if (substr($method, -5) === 'Async')
		{
			$class = $this->getClassName('DigitalPoint\Cloudflare\Http\Promise');
			return new $class(substr($method, 0, -5), $url, $options);
		}
		else
		{
			$client = \XF::app()->http()->client();

			try
			{
				try
				{
					return $client->request($method, $url, $options);
				}
				catch(\GuzzleHttp\Exception\ServerException $e) // HTTP 5xx error, retry once.
				{
					return $client->request($method, $url, $options);
				}
				catch(\GuzzleHttp\Exception\ClientException $e) // Also going to retry once on an HTTP 499 error.
				{
					if ($e->getResponse()->getStatusCode() == 499)
					{
						return $client->request($method, $url, $options);
					}
					throw $e;
				}
			}
			catch(\GuzzleHttp\Exception\ServerException $e)
			{
				$exception = new \DigitalPoint\Cloudflare\Api\Exception\Server($e->getMessage(), $e->getCode(), $e->getPrevious());
				$exception->response = $this->parseResponse($e->getResponse());
				throw $exception;
			}
			catch(\GuzzleHttp\Exception\ClientException $e)
			{
				$exception = new \DigitalPoint\Cloudflare\Api\Exception\Client($e->getMessage(), $e->getCode(), $e->getPrevious());
				$exception->response = $this->parseResponse($e->getResponse());

				if ($exception->response['statusCode'] == 403)
				{
					$error = \json_decode($exception->response['contents'], true);
					if (!empty($error['errors'][0]['code']) && $error['errors'][0]['code'] == 10000) // 10000 authentication error = could be invalid account ID
					{
						$this->updateOption(['cfAccountId' => '', 'cfZoneId' => '']);
					}
					elseif (!empty($error['errors'][0]['code']) && $error['errors'][0]['code'] == 9109) // 9109 = invalid zone ID
					{
						$this->updateOption('cfZoneId', '');
						$this->updateOption('cfZone', '');
					}
				}

				throw $exception;
			}
		}
	}

	public function resolvePromises(array $promises)
	{
		$client = \XF::app()->http()->client();

		foreach ($promises as &$promise)
		{
			$promise = $client->requestAsync($promise->method, $promise->url, $promise->options);
		}

		foreach($promises as &$promise)
		{
			try {
				$response = $promise->wait();

				$body = $response->getBody();
				$content = $body ? $body->getContents() : '';

				$promise = json_decode($content, true);
			}
			catch(\Exception $e)
			{
				$promise = false;
			}
		}
		return $promises;
	}

	protected function parseResponse($response)
	{
		// Response already parsed if it failed > 1 times.  Won't be parsed normally (if it failed 0 or 1 time).
		if (is_array($response))
		{
			return $response;
		}

		$body = $response->getBody();
		return [
			'statusCode' => $response->getStatusCode(),
			'contents' => $body ? $body->getContents() : '',
			'contentType' => $response->getHeaderLine('Content-Type'),
			'contentLength' => $response->getHeaderLine('Content-Length'),
			'lastModified' => $response->getHeaderLine('Last-Modified'),
		];
	}

	protected function logError($message)
	{
		\XF::logError($message);
	}

	protected function phrase($phraseKey, array $params = [])
	{
		return \XF::phrase($phraseKey, $params);
	}
	protected function printableException($message, $code = 0, \Exception $previous = null)
	{
		throw new \XF\PrintableException($message, 0, $previous);
	}

	protected function dateFormat($timestamp, $withTime = false)
	{
		if ($withTime)
		{
			return \XF::phrase('date_x_at_time_y', [
				'date' => \XF::language()->date($timestamp, 'picker'),
				'time' => \XF::language()->time($timestamp)
			])->__toString();
		}
		return \XF::language()->date($timestamp, 'picker');
	}

	protected function getTimeZone()
	{
		return \XF::language()->getTimeZone()->getName();
	}
}