<?php

namespace DigitalPoint\Cloudflare\XF\ControllerPlugin;

class Attachment extends XFCP_Attachment
{
	public function displayAttachment(\XF\Entity\Attachment $attachment, $allowViewLog = true)
	{
		if (\XF::options()->cloudflarePresignAttachments && !empty(\XF::options()->cfR2Bucket['internal-data/attachments']) && $attachment->Data && substr($attachment->Data->getExistingAbstractedDataPath(), 0, 16) === 'internal-data://')
		{
			if (!$attachment->Data)
			{
				return $this->error(\XF::phrase('attachment_cannot_be_shown_at_this_time'));
			}

			$response = $this->app->response();
			$response->setAttachmentFileParams($attachment->filename);

			$contentType = $response->contentType();
			$contentDisposition = $response->header('Content-Disposition');

			$response->removeHeader('Content-Disposition');

			$presignedUrl = $this->repository('DigitalPoint\Cloudflare:Cloudflare')->getPresignedUrl(
				'GET',
				\XF::options()->cfR2Bucket['internal-data/attachments'],
				str_replace('internal-data://', '', $attachment->Data->getExistingAbstractedDataPath()),
				60,
				[
					'response-content-type' => $contentType,
					'response-content-disposition' => $contentDisposition
				]
			);

			return $this->redirect($presignedUrl, null, 'temporary');
		}
		elseif (\XF::options()->cfExternalAttachmentUrl && !empty(\XF::options()->cfR2Bucket['internal-data/attachments']) && $attachment->Data && substr($attachment->Data->getExistingAbstractedDataPath(), 0, 16) === 'internal-data://')
		{
			if (!$attachment->Data)
			{
				return $this->error(\XF::phrase('attachment_cannot_be_shown_at_this_time'));
			}

			$message = '/' . str_replace('internal-data://', '', $attachment->Data->getExistingAbstractedDataPath());
			$time   = time();
			$token  = $time . '-' . urlencode(base64_encode(hash_hmac("sha256", $message . $time, \XF::options()->cfSecretKey, true)));

			return $this->redirect(\XF::options()->cfExternalAttachmentUrl . $message . '?token=' . $token, null, 'temporary');
		}

		return parent::displayAttachment($attachment, $allowViewLog);
	}
}

// ******************** FOR IDE AUTO COMPLETE ********************
if (false)
{
	class XFCP_Attachment extends \XF\ControllerPlugin\Attachment {}
}